if exists('g:gemini_loaded')
  finish
endif

let s:save_cpo = &cpo " save user coptions
set cpo&vim " reset them to defaults

function! GmiHighlights()
  " Handle monospaced blocks
  syn region gmiMono start=/^```/ end=/^```/
  highlight default link gmiMono PreProc

  " Handle between one and three heading levels
  syn match gmiHeader /^#\{1,3}[^#].*$/
  highlight default link gmiHeader Constant

  " Start a link line
  syn match gmiLinkStart /^\[\d\+\]/ nextgroup=gmiLinkUrl skipwhite
  highlight default link gmiLinkStart Special

  " An extremely naive way of handling the URL portion of the link line
  " This is left naive in a deliberate attempt to be unambiguous about
  " what part of a link line gemini considers to be the URL, regardless
  " of whether or not it is a valid URL
  syn match gmiLinkUrl /\S\+/ contained nextgroup=gmiLinkTitle skipwhite
  highlight default link gmiLinkUrl Underlined

  " Skipping whitespace from the URL match all text, including whitespace,
  " until the end of the line
  syn match gmiLinkTitle /.*$/ contained
  highlight default link gmiLinkTitle Comment

  " Handle list items
  syn match gmiListItemPrefix /^\*/
  highlight default link gmiListItemPrefix Statement

  " Handle quotes
  syn match gmiQuoteLine /^>.*/
  highlight default link gmiQuoteLine Comment
endfunction

" command to run our plugin
command! -nargs=1 GeminiOpen lua require('gemini').gemini_open(<f-args>)

let &cpo = s:save_cpo " and restore after
unlet s:save_cpo

let g:gemini_loaded = 1

